import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:stealth_id/stealth_id.dart';
import 'package:stealth_id_example/components/AuthExample.dart';
import 'package:stealth_id_example/components/DocumentExample.dart';
import 'package:stealth_id_example/components/EnrollExample.dart';

void main() {
  runApp(MyApp());
}

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _platformVersion = 'Unknown';

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      platformVersion = await StealthId.platformVersion;
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
        home: DefaultTabController(
      length: 3,
      child: Scaffold(
        appBar: AppBar(
          bottom: TabBar(
            tabs: [
              Tab(icon: Icon(Icons.verified_user)),
              Tab(icon: Icon(Icons.camera)),
              Tab(icon: Icon(Icons.account_box_outlined))
            ],
          ),
          title: const Text('Carespace Stealth Id Example App'),
        ),
        body: Container(
          child: Column(
            children: [
              Expanded(
                flex: 3,
                child: TabBarView(
                  children: [AuthExample(), EnrollExample(), DocumentExample()],
                ),
              ),
              Expanded(
                flex: 1,
                child: Center(
                  child: Text('Running on: $_platformVersion\n'),
                ),
              )
            ],
          ),
        ),
      ),
    ));
  }
}
