//
//  CarespaceClient.swift
//  risecx_stealthId
//
//  Created by Crístofer Domingues on 15/04/21.
//

import Foundation
import SICommunicationSDK

public class CarespaceClient{
    internal var service : CarespaceService
    internal var baseURL : String = ""
    
    init(viewController: FlutterViewController, baseURL:String) {
        self.baseURL = baseURL
        self.service = CarespaceService(viewController:viewController, baseURL: baseURL)
        
    }
    
    public func getHeaderCredential(completionHandler: @escaping (Dictionary<String, Any>) -> Void){
        self.clientRegister { (registerResult) in
            guard let consumerKey = registerResult["consumerKey"], let consumerSecret = registerResult["consumerSecret"] else{
                completionHandler(registerResult)
                return
            }
            self.getAPIToken(consumerKey as? String,consumerSecret as? String){
                (getAPITokenResult) in
                guard let access_token = getAPITokenResult["access_token"] else{
                    completionHandler(getAPITokenResult)
                    return
                }
                completionHandler(["access_token":access_token])
            }
            
        }
    }
    private var basicAuthBase64String:String! {
        let username = "sid-subscriber"
        let password = "!m+Y4:ZDhwwc"
        let basicAuthString = String(format: "%@:%@", username, password)
        guard let basicAuthData = basicAuthString.data(using: .utf8) else {
            return ""
        }
        let basicAuthBase64String = basicAuthData.base64EncodedString()
        return basicAuthBase64String
    }
    
    private func clientRegister(completionHandler: @escaping (Dictionary<String, Any>) -> Void){
        let urlString = "https://carespace-providence-portal.sid.unisys.com/clientRegister"
        let registerHead : [String:String] = ["Authorization":"Basic \(basicAuthBase64String ?? "")", "Content-Type": "application/json"]
        let query : [String:String] = ["customerCode":"providence"]
        
        self.service.webserviceCaller(urlString, requestHeader: registerHead, queryParams: query, method: .get) { (Result) in
            if(Result.keys.contains("consumerKey") && Result.keys.contains("consumerSecret")){
                completionHandler(Result)
            }else{
                completionHandler(["statusCode": "CLIENT_REGISTER_FAIL", "success": false, "message":Result["message"] as? String ?? "Error When Trying to Register Client",  "expectedResult": ""])
            }
        }
    }
    
    private func getAPIToken(_ consumerKey:String?, _ consumerSecret:String?, completionHandler: @escaping (Dictionary<String, Any>) -> Void){
        let url = "https://carespace-providence-portal.sid.unisys.com/getAPIToken"
        let registerHead : [String:String] = ["Authorization":"Basic \(basicAuthBase64String ?? "")", "Content-Type": "application/json", "consumerKey":consumerKey ?? "","consumerSecret": consumerSecret ?? ""]
        let query:[String : String] = ["grant_type" : "client_credentials"]
        self.service.webserviceCaller(url, requestHeader: registerHead,queryParams: query, method: .get) { (Result) in
            if(Result.keys.contains("access_token")){
                completionHandler(Result)
            }else{
                completionHandler(["statusCode": "GET_TOKEN_FAIL", "success": false, "message":Result["message"] as? String ?? "Error When Trying to get API Token",  "expectedResult": ""])
            }
        }
    }
    
    
}
