import Flutter
import UIKit
import SIConfigurationSDK
import SIPlatformSDK
import SIFaceCaptureSDK
import FaceLiveness
import SICommunicationSDK

public class SwiftStealthIdPlugin: NSObject, FlutterPlugin {
internal var viewController : FlutterViewController =  UIApplication.shared.delegate!.window!!.rootViewController as! FlutterViewController
  internal var carespaceClient : CarespaceClient
  internal var baseUrl = "https://carespace-providence-api.sid.unisys.com"
  internal var apiHeader : [String:String] = [
      "Authorization" : "", // Remove when for production
      "Content-Type": "application/json"
   ]

  public override init() {
    self.carespaceClient = CarespaceClient(viewController: viewController, baseURL: baseUrl)
  }
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "stealth_id", binaryMessenger: registrar.messenger())
    let instance = SwiftStealthIdPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
    SIConfigurationHandler.sharedManager().setSelectedCaseloadId(caseloadId: "80264caa-d6ec-4bb6-98dc-9c98d63de322", processName: "enrollment")
   SIPlatformSDKManager.shareManager.setupDefaultConfiguration()
   instance.initiateClient();
  }
    
    private func initiateClient(){
        self.carespaceClient.getHeaderCredential { (headerResult) in
            guard let access_token : String = headerResult["access_token"] as? String else {
                return;
            }
            self.apiHeader.updateValue("Bearer \(access_token)", forKey: "Authorization")
        }
    }


    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
            switch call.method {
                case "getPlatformVersion":
                    result("iOS " + UIDevice.current.systemVersion)
            case "authenticateEnrolledUser":
                self.authenticateEnrolledUser(call, result: result)
            case "enrollUser":
                self.enrollUser(call, result: result)
            case "captureOCR":
                self.captureOCR(call, result: result)
            default:
                result("Unknown Method")
            }
        }
        
    private func enrollUser(_ call: FlutterMethodCall, result: @escaping FlutterResult){
        if let args = call.arguments as? Dictionary<String, Any>,
          let userId = args["userId"] as? String {
            let carespaceFaceEnroll = CarespaceFaceEnroll(viewController: viewController, baseURL: baseUrl, apiHeader: apiHeader);
            carespaceFaceEnroll.enrollUser(userId, result: result);
        } else {
            result(FlutterError.init(code: "BAD_ARGS", message: "Required userId argument is missing.", details: "Required userId argument is missing."))
        }
    }
    
    private func authenticateEnrolledUser(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>,
          let userId = args["userId"] as? String {
            let carespaceFaceAuth = CarespaceFaceAuth(viewController: viewController,baseURL:baseUrl ,apiHeader: apiHeader)
            carespaceFaceAuth.authenticateEnrolledUser(userId, result: result)
        } else {
            result(FlutterError.init(code: "BAD_ARGS", message: "Required userId argument is missing.", details: "Required userId argument is missing."))
        }
    }
    
    
    private func captureOCR(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>,
          let userId = args["userId"] as? String {
            let carespaceDocumentScan = CarespaceDocumentScan(viewController: viewController,baseURL:baseUrl ,apiHeader: apiHeader)
            carespaceDocumentScan.captureOCR(result: result)
        } else {
            result(FlutterError.init(code: "BAD_ARGS", message: "Required userId argument is missing.", details: "Required userId argument is missing."))
        }
    }



}



extension Dictionary {

    

    var jsonStringRepresentation: String? {

        guard let theJSONData = try? JSONSerialization.data(withJSONObject: self, options: [.prettyPrinted]) else {

            return nil

        }

        return String(data: theJSONData, encoding: .utf8)

    }
    
    var toStringDictionary: [String: String]?{
        guard let theJsonStringData = self.jsonStringRepresentation else {
            return nil
        }
        
        if let data = theJsonStringData.data(using: .utf8) {
            do {
                return try JSONSerialization.jsonObject(with: data, options: []) as? [String: String]
            } catch {
                print(error.localizedDescription)
            }
        }
        return nil
        
    }
    
    var toJson: Data? {
        var jsonData = Data.init()
        if JSONSerialization.isValidJSONObject(self){
            do {
                jsonData =  try JSONSerialization.data(withJSONObject: self, options: [])
            }catch{
                print(error.localizedDescription)
                return jsonData
            }
        }
        return jsonData
    }


    mutating func merge(dict: [Key: Value]){

        for (k, v) in dict {

            updateValue(v, forKey: k)

        }

    }

}

extension UIImage {

    

    func toString(imageCompressionRatio : CGFloat = 0.3 ) -> String? { // I am compressing image 0.3 here based on requirement. You also use same compression because iPhone create big size of image.

        let data = self.jpegData(compressionQuality: imageCompressionRatio )

        return data?.base64EncodedString(options:[])

    }

}

 

extension String {

    func toImage() -> UIImage? {

        if let data = Data(base64Encoded: self, options: .ignoreUnknownCharacters){

            return UIImage(data: data)

        }

        return nil

    }

}
